class Messages:
    USER_PROVIDED_CONFIG_HEADER = (
        "All user provided configs for brokers in the cluster are:"
    )
    CLUSTER_CREATE_CONFIRMATION = (
        "Are you sure you want to create the cluster with the saved configuration?"
    )
    CLUSTER_DELETE_CONFIRMATION = "Are you sure you want to delete the cluster?"
    IMAGE_REGISTRY_USER_NAME = "Image registry username"
    IMAGE_REGISTRY_PASSWORD = "Image registry password"


class Errors:
    CONFIG_FILE_SHOULD_BE_PROVIDED = (
        "A configuration file should be provided for connect cluster"
    )
    NOT_A_VALID_URL = "Not a valid URL"
    NO_PLUGIN_TYPE_DETECTED = "No plugin type detected"
