from kfk.commands.acls import acls
from kfk.commands.clusters import clusters
from kfk.commands.configs import configs
from kfk.commands.connect.clusters import clusters
from kfk.commands.connect.connectors import connectors
from kfk.commands.console import console_consumer, console_producer
from kfk.commands.env import env
from kfk.commands.main import kfk
from kfk.commands.operator import operator
from kfk.commands.topics import topics
from kfk.commands.users import users
from kfk.setup import setup

setup()
