SPACE = " "
COLON = ":"
SEMICOLON = ";"
AMPERSAND = "&"
COMMA = ","
EQUALS = "="
NEW_LINE = "\n"
KAFKA_PORT = "9092"
KAFKA_SECURE_PORT = "9093"
KAFKA_SSL = "SSL"
COMMON_NAME_PREFIX = "CN="
BROKER_TMP_FOLDER_PATH = "/tmp"
CONNECT_OUTPUT_TYPE_DOCKER = "docker"
EXTENSION_TAR_GZ = ".tar.gz"
EXTENSION_JAR = ".jar"
EXTENSION_ZIP = ".zip"
TRUE = "true"


class SpecialTexts:
    BROKER_CONFIG_FILE_USER_CONFIG_HEADER = (
        "\n##########\n# User provided configuration\n##########\n"
    )
    CONNECT_BOOTSTRAP_SERVERS = "bootstrap.servers"
    CONNECT_IMAGE = "image"
    CONNECT_PLUGIN_URL = "plugin.url"
    CONNECT_PLUGIN_PATH = "plugin.path"
    CONNECTOR_NAME = "name"
    CONNECTOR_TASKS_MAX = "tasks.max"
    CONNECTOR_CLASS = "connector.class"
    OPERATOR_ROLE_BINDING = "RoleBinding"
    OPERATOR_MY_PROJECT = "myproject"
